@@ -47,6 +47,7 @@ import org.openscience.cdk.isomorphism.UniversalIsomorphismTester;
 import org.openscience.cdk.sgroup.Sgroup;
 import org.openscience.cdk.silent.PseudoAtom;
 import org.openscience.cdk.silent.SilentChemObjectBuilder;
+import org.openscience.cdk.smiles.SmiFlavor;
 import org.openscience.cdk.smiles.SmilesGenerator;
 import org.openscience.cdk.smiles.SmilesParser;
 import org.openscience.cdk.stereo.TetrahedralChirality;
@@ -62,6 +63,7 @@ import java.util.Map;
 
 import static org.hamcrest.CoreMatchers.*;
 import static org.hamcrest.number.IsCloseTo.closeTo;
+import static org.hamcrest.MatcherAssert.assertThat;
 import static org.junit.Assert.*;
 import static org.openscience.cdk.tools.manipulator.AtomContainerManipulator.*;
 
@@ -220,7 +222,7 @@ public class AtomContainerManipulatorTest extends CDKTestCase {
         Assert.assertEquals(6, mol.getAtomCount());
         IAtomContainer ac = AtomContainerManipulator.removeHydrogens(mol);
         Assert.assertEquals(2, ac.getAtomCount());
-        Assert.assertTrue(ac.getFlag(CDKConstants.ISAROMATIC));
+        assertTrue(ac.getFlag(CDKConstants.ISAROMATIC));
     }
 
     @Test
@@ -391,12 +393,12 @@ public class AtomContainerManipulatorTest extends CDKTestCase {
 
         List<String> ids = AtomContainerManipulator.getAllIDs(mol);
         Assert.assertEquals(6, ids.size());
-        Assert.assertTrue(ids.contains("a1"));
-        Assert.assertTrue(ids.contains("a2"));
-        Assert.assertTrue(ids.contains("a3"));
-        Assert.assertTrue(ids.contains("a4"));
-        Assert.assertTrue(ids.contains("a5"));
-        Assert.assertTrue(ids.contains("a6"));
+        assertTrue(ids.contains("a1"));
+        assertTrue(ids.contains("a2"));
+        assertTrue(ids.contains("a3"));
+        assertTrue(ids.contains("a4"));
+        assertTrue(ids.contains("a5"));
+        assertTrue(ids.contains("a6"));
     }
 
     @Test
@@ -693,9 +695,9 @@ public class AtomContainerManipulatorTest extends CDKTestCase {
         IAtomContainer intersection = AtomContainerManipulator.getIntersection(container1, container2);
         Assert.assertEquals(2, intersection.getAtomCount());
         Assert.assertEquals(1, intersection.getBondCount());
-        Assert.assertTrue(intersection.contains(b2));
-        Assert.assertTrue(intersection.contains(o));
-        Assert.assertTrue(intersection.contains(c2));
+        assertTrue(intersection.contains(b2));
+        assertTrue(intersection.contains(o));
+        assertTrue(intersection.contains(c2));
     }
 
     @Test
@@ -742,14 +744,14 @@ public class AtomContainerManipulatorTest extends CDKTestCase {
 
         AtomContainerManipulator.percieveAtomTypesAndConfigureAtoms(container);
         for (IAtom atom : container.atoms()) {
-            Assert.assertTrue(atom.getAtomTypeName() != CDKConstants.UNSET);
-            Assert.assertTrue(atom.getHybridization() != CDKConstants.UNSET);
+            assertTrue(atom.getAtomTypeName() != CDKConstants.UNSET);
+            assertTrue(atom.getHybridization() != CDKConstants.UNSET);
         }
 
         AtomContainerManipulator.clearAtomConfigurations(container);
         for (IAtom atom : container.atoms()) {
-            Assert.assertTrue(atom.getAtomTypeName() == CDKConstants.UNSET);
-            Assert.assertTrue(atom.getHybridization() == CDKConstants.UNSET);
+            assertTrue(atom.getAtomTypeName() == CDKConstants.UNSET);
+            assertTrue(atom.getHybridization() == CDKConstants.UNSET);
         }
     }
 
@@ -1034,7 +1036,7 @@ public class AtomContainerManipulatorTest extends CDKTestCase {
         mol = AtomContainerManipulator.createAllCarbonAllSingleNonAromaticBondAtomContainer(mol);
         String smiles2 = "C1CCCCC1";
         IAtomContainer mol2 = sp.parseSmiles(smiles2);
-        Assert.assertTrue(new UniversalIsomorphismTester().isIsomorph(mol, mol2));
+        assertTrue(new UniversalIsomorphismTester().isIsomorph(mol, mol2));
     }
 
     @Test
@@ -1052,7 +1054,7 @@ public class AtomContainerManipulatorTest extends CDKTestCase {
 
         IAtomContainer anonymous = AtomContainerManipulator.anonymise(cyclohexane);
 
-        Assert.assertTrue(new UniversalIsomorphismTester().isIsomorph(anonymous, TestMoleculeFactory.makeCyclohexane()));
+        assertTrue(new UniversalIsomorphismTester().isIsomorph(anonymous, TestMoleculeFactory.makeCyclohexane()));
 
         assertThat(anonymous.getAtom(0).getSymbol(), is("C"));
         assertThat(anonymous.getAtom(2).getSymbol(), is("C"));
@@ -1109,7 +1111,7 @@ public class AtomContainerManipulatorTest extends CDKTestCase {
             if (expected == null)
                 assertNull(actual);
             else
-                Assert.assertThat(actual,
+                org.hamcrest.MatcherAssert.assertThat(actual,
                                   is(closeTo(expected, 0.001)));
         }
     }
@@ -1288,6 +1290,18 @@ public class AtomContainerManipulatorTest extends CDKTestCase {
                       "CCC[H] |Sg:n:2:n:ht|");
     }
 
+    @Test
+    public void keepStereoGroup() throws Exception {
+        assertRemoveH("C[C@]([H])(O)CC |o1:1|",
+                      "C[C@H](O)CC |o1:1|");
+    }
+
+    @Test
+    public void keepStereoGroup2() throws Exception {
+        assertAddH("C[C@H](O)CC |o1:1|",
+                   "C([C@](O[H])(C(C([H])([H])[H])([H])[H])[H])([H])([H])[H] |o1:1|");
+    }
+
     @Test
     public void molecularWeight() throws InvalidSmilesException, IOException {
         SmilesParser smipar = new SmilesParser(SilentChemObjectBuilder.getInstance());
@@ -1330,7 +1344,16 @@ public class AtomContainerManipulatorTest extends CDKTestCase {
         SmilesParser smipar = new SmilesParser(SilentChemObjectBuilder.getInstance());
         IAtomContainer m = smipar.parseSmiles(smiIn);
 
-        String smiAct = new SmilesGenerator().create(AtomContainerManipulator.removeHydrogens(m));
+        String smiAct = new SmilesGenerator(SmiFlavor.Default).create(AtomContainerManipulator.removeHydrogens(m));
+
+        assertThat(smiAct, is(smiExp));
+    }
+
+    static void assertAddH(String smiIn, String smiExp) throws Exception {
+        SmilesParser smipar = new SmilesParser(SilentChemObjectBuilder.getInstance());
+        IAtomContainer m = smipar.parseSmiles(smiIn);
+        AtomContainerManipulator.convertImplicitToExplicitHydrogens(m);
+        String smiAct = new SmilesGenerator(SmiFlavor.Default).create(m);
 
         assertThat(smiAct, is(smiExp));
     }
@@ -1364,6 +1387,19 @@ public class AtomContainerManipulatorTest extends CDKTestCase {
                    closeTo(4729.147, 0.001));
     }
 
+    @Test public void getMassBadIsotopes() throws InvalidSmilesException {
+        IChemObjectBuilder bldr = SilentChemObjectBuilder.getInstance();
+        IAtomContainer mol = new SmilesParser(bldr).parseSmiles("[24cH]1[24cH][24cH][24cH][24cH][24cH]1");
+        assertThat(AtomContainerManipulator.getMass(mol, MolWeight),
+                closeTo(150.0476, 0.001));
+        assertThat(AtomContainerManipulator.getMass(mol, MolWeightIgnoreSpecified),
+                closeTo(78.1120, 0.001));
+        assertThat(AtomContainerManipulator.getMass(mol, MonoIsotopic),
+                closeTo(150.0469, 0.001));
+        assertThat(AtomContainerManipulator.getMass(mol, MostAbundant),
+                closeTo(150.0469, 0.001));
+    }
+
     @Test public void getMassCranbinSpecIsotopes() {
         IChemObjectBuilder bldr = SilentChemObjectBuilder.getInstance();
         IAtomContainer mol =
@@ -1393,4 +1429,20 @@ public class AtomContainerManipulatorTest extends CDKTestCase {
         assertThat(AtomContainerManipulator.getMass(mol, MostAbundant),
                    closeTo(4731.154, 0.001));
     }
+
+    // can't put these test in cdk-formula since we can't access SMILES and it's a bit verbose
+    // to construct the molecules as needed
+    @Test public void getFormulaMultiattach() throws InvalidSmilesException {
+        SmilesParser smipar = new SmilesParser(SilentChemObjectBuilder.getInstance());
+        IAtomContainer mol = smipar.parseSmiles("[Ru]([P](CCC1=CC=CC=C1)(C2CCCCC2)C3CCCCC3)(Cl)(Cl)*.C1(=CC=C(C=C1)C(C)C)C |m:24:25.26.27.28.29.30|");
+        String mf = MolecularFormulaManipulator.getString(MolecularFormulaManipulator.getMolecularFormula(mol));
+        assertThat(mf, CoreMatchers.is("C30H45Cl2PRu"));
+    }
+
+    @Test public void getFormulaAttach() throws InvalidSmilesException {
+        SmilesParser smipar = new SmilesParser(SilentChemObjectBuilder.getInstance());
+        IAtomContainer mol = smipar.parseSmiles("*c1cc(*)ccc1 |$_AP1;;;;R;$|");
+        String mf = MolecularFormulaManipulator.getString(MolecularFormulaManipulator.getMolecularFormula(mol));
+        assertThat(mf, CoreMatchers.is("C6H4R"));
+    }
 }
